@@ -1,23 +1,38 @@
-"""
-FlipConv2d is a special Conv2d layer whose result is invariant against horizontal & vertical flipping
-"""
-
 import torch
 from torch import nn
-from torch.nn.functional import conv2d
+from torch.nn.functional import conv2d as _conv2d
 
 
 class FlipConv2d(nn.Module):
     """ FlipConv2d is special case of the standard nn.Conv2d. They differs in the kernal structure.
-    This kernel structure enables making the layer "invariant" with respect to flipping. 
+    
+    This kernel structure enables making the layer invariant to flipping. 
+    Formally, if the feature map X' is the horizonatally-flipped version of X 
+    and `flipconv2d` is this layer when `h_invariant` is true, then
+    flipconv2d(X`) equals the horizonatally-flipped version of flipconv2d(X).
+    This is achieved by forcing the kernel plane (indexed by the last two dimensions) to be symmetric with respect to its vertical axis.
+
+    This module accepts the same parameters as nn.Conv2d, as well as `h_invariant` & `v_invariant`.
+
+    if `h_invariant` and `v_invariant` are both false, this layer acts like nn.Conv2d.
 
     Args:
-        - h_invariant: If true, the kernels are horizonatally symmetrical.
-        - v_invariant: If true, the kernels are vertically symmetrical.
-        - **kwargs: Arguments used for the standard nn.Conv2d layer.
+        h_invariant (bool, Optional): If true, the kernels are symmetric with respect to vertical axis. Default: True.
+        v_invariant (bool, Optional): If true, the kernels are symmetric with respect to horizontal axis. Default: True.
+        **kwargs: Arguments passed for the standard nn.Conv2d layer.   
+                    Note that `in_channels`, `out_channels` & `kernel_size` are required. The others are optional and take the default value of nn.Conv2d.
 
-    Forward:
-        - x (Tensor): its shape is [B, C, H, W]
+    Shape:
+        - x (torch.Tensor): A [B, C, H, W] input feature map.
+
+    Attributes:
+        kernel (torch.Tensor): The kernel tensor used for the cross-correlation operation.
+
+    Example:
+        >>> from deepblocks.layer import FlipConv2d
+        >>> conv = FlipConv2d(v_invariant=False, in_channels=2, out_channels=4, kernel_size=5, stride=2, bias=False)
+        >>> x = torch.rand(10, 2, 32, 32)
+        >>> fm = conv(x)
 
     """
     def __init__(self, h_invariant: bool = True, v_invariant: bool = True, **kwargs):
@@ -59,6 +74,8 @@ class FlipConv2d(nn.Module):
 
     @property
     def kernel(self):
+        """ Returns kernel weights used for the Conv2d operation.
+        """
         kernel = self._kernel
 
         # Flip the kernel vertically
@@ -73,7 +90,6 @@ class FlipConv2d(nn.Module):
 
         return kernel
 
-    def forward(self, x):
-        kernel = self.kernel
-
-        return conv2d(x, kernel, bias=self.bias, **self.kwargs)
\ No newline at end of file
+    def forward(self, x:torch.Tensor):
+        """ """
+        return _conv2d(x, self.kernel, bias=self.bias, **self.kwargs)
\ No newline at end of file
